package sorting;
/**
 *  Algorithms.java  28 March 2010 
 */

import edu.neu.ccs.util.*;
import edu.neu.ccs.console.*;

import java.util.*;

import tester.*;

public class Algorithms implements ConsoleAware{

  /** the data for all <code>City</code> data
   *  saved as <code>ArrayList</code> 
   */
  ArrayList<City> cities = new ArrayList<City>();
  
  /** the data for all <code>City</code> data
   *  saved as <code>ILo</code> 
   */
  ILo<City> citylist = new MtLo<City>();

  //-------------------------------------------------------//
  //------------ DATA INITIALIZATION SECTION --------------//
  //-------------------------------------------------------//
  
  /** 
   * Initialize cities with the data from a file 
   *  @param limit the maximum number of item to read
   *  */
  public void initCities(int limit){
    Traversal<City> tr = new InFileCityTraversal();

    System.out.println("Reading " + limit + " items of city data");
    int i = 0;
    try{
      while (!tr.isEmpty() && i < limit){
        City c = tr.getFirst();
        this.cities.add(c);
        this.citylist = new ConsLo<City>(c, this.citylist);
        tr= tr.getRest();
        if (this.cities.get(i).name.contains("9"))
          System.out.println(this.cities.get(i).toString());
        i++;
      }
    }
    catch(IllegalUseOfTraversalException e){
      System.out.println(
          "IllegalUseOfTraversalException: cannot happen" +
          e.getMessage());
    } 
  }
  
  /**
   * Produce a reversed list of cities 
   * from <code>this.cities</code>
   * @return reversed city list
   */
  ArrayList<City> reverse(){
    ArrayList<City> result = new ArrayList<City>();
    for (int i = this.cities.size() - 1; i >= 0; i--){
      result.add(this.cities.get(i));
    }
    return result;
  }
  
  /**
   * Produce a copy of the list of cities 
   * from <code>this.cities</code>
   * @return a copy of the city list
   */
  ArrayList<City> copy(){
    ArrayList<City> result = new ArrayList<City>();
    for (City c: this.cities){
      result.add(c);
    }
    return result;
  }

  /**
   * Produce a randomized list of all cities
   * @return randomized <code>ArrayList</code> of all <City> data.
   */
  public ArrayList<City> randomData(){
    // make a copy of the original data
    ArrayList<City>  temp = this.copy();
    ArrayList<City>  result = new ArrayList<City> ();
    
    int index = 0;
    for (int i = temp.size() - 1; i >= 0; i--){
      // start copying elements random index from the start
      index = MathUtilities.randomInt(0, i);
        result.add(temp.remove(index));    
    }
    console.out.println("Data set of size " + result.size() + " initialized.");
    return result;
  }
  
  
	
	
	/**
	 * Sort the given list using selection sort
	 * @param <T>
	 * @param alist
	 * @param comp
	 * @return
	 */
	public <T> void selectionsort(ArrayList<T> alist, Comparator<T> comp){		
		int i = 0;
		while (i < alist.size() - 1){
			swap(alist, i, findFirstLoc(alist, comp, i));
			i = i + 1;
		}				
	}
	
	// swap the items at the given locations in the given array list
	public <T> void swap(ArrayList<T> a, int i, int j){
		T temp = a.get(i);
		temp = a.set(j, temp);
		a.set(i, temp);
	}
	
	// find the location of the least element in the given alist
	// starting at the curr location, using the given comp
	// assume curr < alist.size() - 2
	public <T> int findFirstLoc(ArrayList<T> alist, Comparator<T> comp, int curr){
		int loc = curr;
		T minObj = alist.get(loc);
		int index = curr + 1;
		while (index < alist.size()){
			if (comp.compare(minObj, alist.get(index)) > 0){
				loc = index;
				minObj = alist.get(index);
			}
			index = index + 1;
		}
		return loc;
	}
}

