package sorting;

import edu.neu.ccs.*;	    // to get XInt
import java.util.*;       // to get StringTokenizer
import java.text.*;       // to get ParseException

/**
 * @(#)City.java  1.0  28 March 2010
 * A simple class to use for learning about classes and objects.
 *
 * @author Viera K. Proulx
 */
class City implements Stringable{
    
    /*-------------------------------------------------------------------------
     Member data 
     *-----------------------------------------------------------------------*/
    
    /** The zip code for this city. */
    public int zip;
    
    /** The name of this city. */
    public String name;
    
    /** The state for this city */
    public String state;
    
    /* The longitude for this city */
    public double longitude;

    /* The latitude for this city */
    public double latitude;

   /* Decimal format to print leading zeros in zip code */
    public static DecimalFormat zipFormat = new DecimalFormat("00000");

    /*-------------------------------------------------------------------------
     Constructor
     *-----------------------------------------------------------------------*/
    
    /** The full constructor */
    public City (int zip, String name, String state, 
    		         double longitude, double latitude){
        this.zip   = zip;
        this.name  = name;
        this.state = state;
        this.longitude = longitude;
        this.latitude  = latitude;
    } 
    
    /** A copy constructor */
    public City(City aCity){
        this(aCity.zip, aCity.name, aCity.state, aCity.longitude, aCity.latitude);
    }
    
    /**
     * A convenience constructor that gets the zip code as a <code>String</code>
     * @param zip a <code>String</code> that represents the zip code
     * @param name the name of this city
     * @param state the state as a two letter abbreviation (all caps)
     * @param longitude the longitude to six decimals
     * @param latitude the latitude to six decimals
     */
    public City (String zip, String name, String state, 
    		double longitude, double latitude){
        this.zip = this.decimal(zip);
        this.name  = name;
        this.state = state;
        this.longitude = longitude;
        this.latitude  = latitude;
    }
    
    /** the default constructor that initializes a fake city */
    public City(){
        this.zip   = 48170;
        this.name  = "Plymouth";
        this.state = "MI";
        this.longitude  = 83.479946;
        this.latitude = 42.368820;
    }
       
    /*-------------------------------------------------------------------------
     Stringable interface
     *-----------------------------------------------------------------------*/
    /** Convert the city data to a well-structured string for file output*/
    public String toStringData(){
    	String s = "" + City.zipFormat.format(this.zip) + ", " +
    	           this.name  + ", " +
    	           this.state + ", " +
    	           this.longitude + ", " +
			   this.latitude  + "\n";
    	return s;
    }
        
    /** Initialize all fields of this city from a well-formed string */
    public void fromStringData(String s) throws java.text.ParseException{

        try{
            /* set up the string tokenizer to extract city data */
            StringTokenizer st = new StringTokenizer(s, ",");

            this.zip       = decimal(st.nextToken());
            this.name      = st.nextToken();
            this.state     = st.nextToken();
            this.longitude = doubleValue(st.nextToken());
            this.latitude  = doubleValue(st.nextToken());
        }
        catch(NoSuchElementException e){
            System.out.println(e.getMessage());
            throw new java.text.ParseException("Tokenizer Error", 0);
        }
    }
    
    /** convert a String to int - if possible */
    protected int intValue(String s) throws ParseException{
        return (new XInt(s)).intValue();
    }    
    
    /** convert a String to double - if possible */
    protected double doubleValue(String s) throws ParseException{
    		return (new XDouble(s)).doubleValue();
    }
    
    /*-------------------------------------------------------------------------
     Console print
     *-----------------------------------------------------------------------*/
    /** print the city data */
    public String toString(){
    	return ("new " + getClass().getName() + "(" + 
    	        City.zipFormat.format(this.zip)   + ", " + 
    	        this.name  + ", " + 
    	        this.state + ", " + 
    	        this.longitude + ", " + 
		    this.latitude + ")\n");
    }
    
    /** helper method to convert zip code string to integer */
    public int decimal(String s){
    
    	char[] c = s.toCharArray();
    	int n = 0;
    	try {
    	for (int i = 0; i < 5; i++){
    		n = 10 * n + intValue("" + c[i]);
    	}
    	}
    	catch(ParseException e){
    		System.out.println("Converting zip to int " + e);
    	}
    	return n;
    }
    
}
