package sorting;

import java.util.*;

import tester.*;

/**
 * The test suite for the Sorting Algorithms exploration
 * 
 * @author Viera K. Proulx
 *
 */
public class ExamplesSorting{
  ExamplesSorting(){}
  
  /** data for the methods and tests */
  ArrayList<City> NEcities = new ArrayList<City>();
  ArrayList<City> NEcitiesSorted = new ArrayList<City>();
  
  City concord = new City("03301", "Concord", "NH", 71.527734, 43.218525);
  City montpellier = new City("05602", "Montpelier", "VT", 72.576992, 44.264082);
  City augusta = new City("04330", "Augusta", "ME", 69.766548, 44.323228);
  City boston = new City("02115", "Boston", "MA", 71.092215, 42.342706);
  City hartford = new City("06120", "Hartford", "CT", 72.675807, 41.78596);
  City providence = new City("02908", "Providence", "RI", 71.437684, 41.838294);

  ILo<City> NEcitiesList = 
    new ConsLo<City>(this.concord,
        new ConsLo<City>(this.montpellier,
            new ConsLo<City>(this.augusta,
                new ConsLo<City>(this.boston,
                    new ConsLo<City>(this.hartford,
                        new ConsLo<City>(this.providence,
                            new MtLo<City>()))))));

  ILo<City> NEcitiesListSortedNames = 
    new ConsLo<City>(this.augusta,
        new ConsLo<City>(this.boston,
            new ConsLo<City>(this.concord,
                new ConsLo<City>(this.hartford,
                    new ConsLo<City>(this.montpellier,
                        new ConsLo<City>(this.providence,
                            new MtLo<City>()))))));
  
  ArrayList<City> NEcitiesArrList = new ArrayList<City>();
  ArrayList<City> NEcitiesArrListSortedNames = new ArrayList<City>();
  
  void initNE(){
    this.NEcitiesArrList.clear();
    this.NEcitiesArrList.add(this.concord);
    this.NEcitiesArrList.add(this.montpellier);
    this.NEcitiesArrList.add(this.augusta);
    this.NEcitiesArrList.add(this.boston);
    this.NEcitiesArrList.add(this.hartford);
    this.NEcitiesArrList.add(this.providence);
    
    this.NEcitiesArrListSortedNames.clear();
    this.NEcitiesArrListSortedNames.add(this.augusta);
    this.NEcitiesArrListSortedNames.add(this.boston);
    this.NEcitiesArrListSortedNames.add(this.concord);
    this.NEcitiesArrListSortedNames.add(this.hartford);
    this.NEcitiesArrListSortedNames.add(this.montpellier);
    this.NEcitiesArrListSortedNames.add(this.providence);
        
  }
  
  Comparator<City> nameComp = new Comparator<City>(){
    public int compare(City c1, City c2){
      if (c1.name.compareTo(c2.name) == 0)
        return c1.state.compareTo(c2.state);
      else
        return c1.name.compareTo(c2.name);
    }
  };
  
  Comparator<City> zipComp = new Comparator<City>(){
    public int compare(City c1, City c2){
      return c1.zip - c2.zip;
    }
  };
  
  Algorithms algo = new Algorithms();
  
  /** generate lists of neighboring states */
  public void makeNewEngland(){
    this.NEcities.add(this.concord);
    this.NEcities.add(this.montpellier);
    this.NEcities.add(this.augusta);
    this.NEcities.add(this.boston);
    this.NEcities.add(this.hartford);
  }  
  
  /** generate lists of neighboring states */
  public void makeSortedNewEngland(){
    this.NEcitiesSorted.add(this.augusta);
    this.NEcitiesSorted.add(this.boston);
    this.NEcitiesSorted.add(this.concord);
    this.NEcitiesSorted.add(this.hartford);
    this.NEcitiesSorted.add(this.montpellier);
  }

  ArrayList<City> cities = new ArrayList<City>();
  
  ILo<City> citylist = new MtLo<City>();

  /** 
   * Initialize cities with the data from a file 
   *  @param limit the maximum number of item to read
   *  */
  public void initCities(int limit){
    Traversal<City> tr = new InFileCityTraversal();

    System.out.println("Reading " + limit + " items of city data");
    int i = 0;
    try{
      while (!tr.isEmpty() && i < limit){
        City c = tr.getFirst();
        this.cities.add(c);
        this.citylist = new ConsLo<City>(c, this.citylist);
        tr= tr.getRest();
        if (this.cities.get(i).name.contains("9"))
          System.out.println(this.cities.get(i).toString());
        i++;
      }
    }
    catch(IllegalUseOfTraversalException e){
      System.out.println(
          "IllegalUseOfTraversalException: cannot happen" +
          e.getMessage());
    } 
  }
  
  CityConverter cityConvert = new CityConverter();
 
  /**
   * The test driver
   */
  public void testInitData(Tester t){
    this.makeNewEngland();
    
    //initCities(200);
    //System.out.println("We have " + this.cities.size() + " cities");
    for (City c: this.NEcities){
      System.out.println(c.toString());
    }
    
    this.cityConvert.testScanner();
    
    this.cities.clear();
  }
  
  /**
   * test the quicksort for cities
   */
  public void testQuickSort(Tester t){
    t.checkExpect(
        this.NEcitiesList.quicksort(this.nameComp), 
        this.NEcitiesListSortedNames);
    
    this.NEcities.clear();
    makeNewEngland();
    makeSortedNewEngland();
    this.algo.selectionsort(this.NEcities, this.nameComp);
    t.checkExpect(this.NEcities, this.NEcitiesSorted);
  }
    /**
     * test the a lot of stuff for cities
     */
  
  /*
    public void testAlot(Tester t){
    
    initCities(25000);
    //System.out.println("The first fifty cities in the list: ");  
    //for (int i = 0; i < 50; i++){
    //  System.out.println(this.cities.get(i).toString());
    //}
    

    // run the timed test and return the measured time
    long startTime = (new Date()).getTime();
    //this.algo.quickSort(this.cities, this.nameComp);
    this.algo.selectionsort(this.cities, this.zipComp);
    long endTime = (new Date()).getTime();
    System.out.println("Time for quicksort " + (endTime - startTime));
    
    System.out.println("We have sorted " + this.cities.size() + " cities");  
    for (int i = 0; i < 50; i++){
      System.out.print(this.cities.get(i).toString());
    }
    
    System.out.println("" + (this.citylist instanceof MtLo));

    //ILo<City> sortedcitylist = 
    //  this.citylist.quicksort(this.nameComp);
    
 // run the timed test and return the measured time
    startTime = (new Date()).getTime();

    ILo<City> sortedcitylist = 
      this.citylist.insertionsort(this.nameComp);

    ILo<City> sortedcitylist2 = 
      this.citylist.quicksort(this.nameComp);

    endTime = (new Date()).getTime();
    System.out.println("Time for quicksort " + (endTime - startTime));
    
    
    System.out.println("" + (sortedcitylist instanceof MtLo));

    System.out.println("We have sorted " + this.cities.size() + " cities");  
    System.out.println(sortedcitylist.isSorted(this.nameComp));    
    System.out.println(sortedcitylist2.isSorted(this.nameComp));    
  }
  
*/
  
  
  // test insertSorted
  public void testSelectionSort(Tester t){
    
    Examples e = new Examples();
    
    ArrayList<City> source = new ArrayList<City>();
    source.add(e.duluth1);
    source.add(e.northville);
    source.add(e.plymouth);
    
    ArrayList<City> unsorted = new ArrayList<City>();
    unsorted.add(e.northville);
    unsorted.add(e.plymouth);
    unsorted.add(e.duluth1);
    
    ArrayList<City> sorted = new ArrayList<City>();
    sorted.add(e.duluth1);
    sorted.add(e.northville);
    sorted.add(e.plymouth);
    
    // EFFECT
    this.algo.selectionsort(unsorted, this.nameComp);
    t.checkExpect(sorted, unsorted, "selectionSort");
               
   }
  
  // an arraylist of sorting algorithms to test 
  SortAlgorithm<City> bst   = new SortingBST<City>();
  SortAlgorithm<City> insertlist  = new SortingInsList<City>();
  SortAlgorithm<City> selectarray = new SortingSelection<City>();
  SortAlgorithm<City> quicklist   = new SortingQuicksortList<City>();
  SortAlgorithm<City> insertarray = new SortingIns<City>();
  SortAlgorithm<City> quickarray  = new SortingQuicksort<City>();
  SortAlgorithm<City> heapsort  = new student.Heapsort<City>();
  
  void testAlgos(Tester t){
    this.quicklist.init(this.NEcitiesArrList);
    t.checkExpect(this.quicklist.sort(this.nameComp), 
        this.NEcitiesArrListSortedNames);
  }

}