package sorting;


import edu.neu.ccs.console.*;
import edu.neu.ccs.gui.*;
import tester.*;
/**
 * A functional (External) iterator 
 * for console input of a list of cities
 * @author Viera K. Proulx
 */
class InConsoleTraversal implements Traversal<City>, ConsoleAware {

  /*-------------------------------------------------------- 
  Member data
  *------------------------------------------------------*/
  /** the <code>City</code> object for reading in the data */
  protected City c;

  /** is there anything available to read? */
  protected boolean done = true;
  
  
  protected boolean submitted = false;
    
 /*-------------------------------------------------------- 
  Constructor
  *------------------------------------------------------*/
  /**
   * Starting constructor - requests the first city data
   * from the user.
   */
  InConsoleTraversal() { 

    this.done = false;
    read();          
  }

  /**
   * Internally built constructor - sets <code>done</code> to false
   * after the user declined to provide more data
   * @param done is there more data available to read?
   */
  InConsoleTraversal(boolean done) { 

    this.done = done;          
  }
  
  /*--------------------------------------------------------
  Methods to implement the Traversal interface
  *------------------------------------------------------*/

  /**
   * Produce the <code>Traversal</code> for the next city
   */
  public Traversal<City> getRest(){
    if (!this.isEmpty())
      return new InConsoleTraversal();
    else
      return new InConsoleTraversal(false);
  }

  /**
   * Produce the city that has been read 
   */
  public City getFirst() {
    return this.c;
  }

  /**
   * Return true if no more data is to be read
   */
  public boolean isEmpty() {
    return (this.done); 
  }
 
 /*--------------------------------------------------------------
  Methods
  *------------------------------------------------------------*/

 /**
  * Read data for one city from the console.
  * If the user declines to provide more data, 
  * set <code>done</code> to true, indicating the end of input. 
  */
 public City read(){
     
     try{
       this.c = new City
         (console.in.requestInt("Zip code: "),
          console.in.requestString("City name: "),
          console.in.requestString("State: "),
          console.in.requestDouble("Longitude: "),
		 console.in.requestDouble("Latitude: "));
     }
     catch(CancelledException e){
     	console.out.println("End of input");
     	this.done = true;
     }
     return this.c;
 }
 
}
     
