package sorting;
import java.util.*;
import tester.*;

/**
 * A class that converts Java <code>Iterator</code> into the 
 * functional iterator specified by the <code>Traversal</code>
 * interface.
 * @author Viera K. Proulx
 *
 * @param <T> the type of data generated by the iterators
 */
public class MyIterator<T> implements Traversal<T>{
	
  /** the given iterator that generates the data to produce */
	Iterator<T> iter;
	
	/** a helper iterator needed to look ahead */
	Iterator<T> nextIt;
	
	/** the object to be generated by the traversal */
	T obj;
	
	/** an indicator whether more items are available */
	boolean more;
	
	/**
	 * The constructor that initializes the <code>Traversal</code>
	 * @param iter the given <code>Iterator</code>
	 */
	public MyIterator(Iterator<T> iter){
		this.iter = iter;
		this.nextIt = advance(this.iter);
	}
	
	/** 
	 * Save the current item and advance the iterator
	 * or mark that no more items are available
	 * @param it the iterator that is generating the data
	 * @return
	 */
	public Iterator<T> advance(Iterator<T> it){
		if (it.hasNext()){
			this.more = true;
			this.obj = it.next();
			return it;
		}
		else{
			this.obj = null;
			this.more = false;
			return null;
		}	
	}
	
	/**
	 * Are there any items available to read?
	 */
	public boolean isEmpty(){
		return !this.more;
	}
	
	/**
	 * Produce the next item available to read.
	 * Notice that repeated requests deliver the same item 
	 * -- as expected.
	 */
	public T getFirst(){
		if (this.more){
			return this.obj;
		}
		else {
			throw new NoSuchElementException("no current for empty iterator");
		}
	}
	
	/**
	 * Produce a new traversal poised to deliver the next available item
	 * or empty if thre is no data left to read
	 */
	public Traversal<T> getRest(){
		if (this.more){
			return new MyIterator<T>(this.nextIt);		
		}
		else{
			throw new NoSuchElementException("no next for empty iterator");
		}
	}
}