package sorting;

import java.util.*;

////////////////////////////////////////////////////////////////
// SortAlgorithm interface
////////////////////////////////////////////////////////////////
/**
 * An interface for testing sorting algorithms. 
 * The <code>init</code> method copies the given data into the format 
 * needed for this implementation, the <code>sort</code> method 
 * is then timed.
 */
public interface SortAlgorithm<T> {
  
  /**
   * Initialize the list to be sorted from the given data.
   * @param alist the <code>ArrayList</code> to sort
   */
  public void init(ArrayList<T> alist); 
  
  /**
   * Sort the given <code>ArrayList</code> using the given 
   * <code>Comparator</code>. Return the sorted result.
   * For <em>in-place</em> algorithms <code>alist</code> should be
   * the result.
   * @param alist the <code>ArrayList</code> to sort
   * @param comp the <code>Comparator</code> that determines the ordering
   * @return the sorted <code>ArrayList</code>
   */
	public ArrayList<T> sort(Comparator<T> comp);
}

