package sorting;
import java.util.*;

/**
 * One of the several sorting algorithms
 * to use for sorting detective work
 * @author Viera K. Proulx
 *
 * @param <T> the type of data to be sorted
 */
class SortingBST<T> implements SortAlgorithm<T>{

  /** The <code>ArrayList</code> to sort */
  public ArrayList<T> alist;
  
  /**
   * Initialize the list to be sorted from the given data.
   * @param alist the <code>ArrayList</code> to sort
   */
  public void init(ArrayList<T> alistin){
    this.alist = new ArrayList<T>();
    for (T t: alistin){
      this.alist.add(t);
    }
  }
  
  /**
   * Sort the given <code>ArrayList</code> using the given 
   * <code>Comparator</code>. Return the sorted result.
   * For <em>in-place</em> algorithms <code>alist</code> should be
   * the result.
   * @param alist the <code>ArrayList</code> to sort
   * @param comp the <code>Comparator</code> that determines the ordering
   * @return the sorted <code>ArrayList</code>
   */
  public ArrayList<T> sort(Comparator<T> comp){
    return this.BSTsort(comp);
  }

	
	// Binary tree sort for AListOfObject
	public ArrayList<T> BSTsort(Comparator<T> comp){
	  ArrayList<T>  result = new ArrayList<T>();
		ABST<T> citiesTree = buildTree(comp, new Leaf<T>());
		return citiesTree.inorder(result);
	}
	
	// build a BST from the data given by the IRange and 
	// using the given Comparator
	public ABST<T> buildTree(Comparator<T> comp, ABST<T> tree){
		for (T t: this.alist){
			tree = tree.insert(t, comp);
		}
		return tree;		
	}
}


// to represent a binary search tree
abstract class ABST<T> {

  // insert the given city into this BST
  abstract ABST<T> insert(T t, Comparator<T> comp);

  // produce an ArrayList from this BST and the given ArrayList
  abstract ArrayList<T> inorder(ArrayList<T> inlist);

/*
  // ** DRAFT TEMPLATE ** Edit as needed.
  // purpose statement 
  abstract ??? mmm();
*/
}


// to represent a leaf node of a BST
class Leaf<T> extends ABST<T> {

  Leaf() { }
  
  // insert the given city into this BST
  ABST<T> insert(T t, Comparator<T> comp){
    return new Node<T>(t, this, this);
  }

  // produce an ArrayList from this BST and the given ArrayList
  ArrayList<T> inorder(ArrayList<T> inlist){
    return inlist;
  }

/*
  // ** DRAFT TEMPLATE ** Edit as needed.
  ??? mmm() {
  }
*/
}

// to represent an internal node of a BST
class Node<T> extends ABST<T> {
  T data;
  ABST<T> left;
  ABST<T> right;

  Node(T data, ABST<T> left, ABST<T> right) {
    this.data = data;
    this.left = left;
    this.right = right;
  }
  
  // insert the given item into this ABST
  ABST<T> insert(T t, Comparator<T> comp){
    if (comp.compare(t, this.data) < 0)
      return 
      new Node<T>(this.data, 
                  this.left.insert(t, comp), 
                  this.right);
    else 
      return 
      new Node<T>(this.data, 
                  this.left, 
                  this.right.insert(t, comp));
  }

  // produce an ArrayList from this ABST and the given ArrayList
  ArrayList<T> inorder(ArrayList<T> inlist){
    inlist = this.left.inorder(inlist);
    inlist.add(this.data);
    inlist = this.right.inorder(inlist);
    return inlist;
  }

/*
  // ** DRAFT TEMPLATE ** Edit as needed.
  ??? mmm() {
    ... this.left.mmm() ...
    ... this.right.mmm() ...
  }
*/
}
