package sorting;
import java.util.*;

/**
 * One of the several sorting algorithms
 * to use for sorting detective work
 * @author Viera K. Proulx
 *
 * @param <T> the type of data to be sorted
 */
class SortingIns<T> implements SortAlgorithm<T>{

  /** The <code>ArrayList</code> to sort */
  public ArrayList<T> alist;
  
  /**
   * Initialize the list to be sorted from the given data.
   * @param alist the <code>ArrayList</code> to sort
   */
  public void init(ArrayList<T> alistin){
    this.alist = new ArrayList<T>();
    for (T t: alistin){
      this.alist.add(t);
    }
  }
  
	
  /**
   * Sort the given <code>ArrayList</code> using the given 
   * <code>Comparator</code>. Return the sorted result.
   * For <em>in-place</em> algorithms <code>alist</code> should be
   * the result.
   * @param alist the <code>ArrayList</code> to sort
   * @param comp the <code>Comparator</code> that determines the ordering
   * @return the sorted <code>ArrayList</code>
   */
  public ArrayList<T> sort(Comparator<T> comp){
    return this.insertSort(this.alist, comp);
  }

	
	// insertion sort for ArrayList
	public ArrayList<T> insertSort(ArrayList<T> source, 
	                               Comparator<T> comp){
		ArrayList<T> result = new ArrayList<T>();
		int index = 0;
		
		while (index < source.size()){
			result = insertSorted(source.get(index), 
					              result, 
								 comp);
			index = index + 1;
		}
		return result;
	}	
	
	// insert the given object into the given sorted list
    public ArrayList<T> insertSorted(T t, ArrayList<T> result, 
        Comparator<T> comp){
    		int loc = 0;
    		while ((loc < result.size()) &&
    			  (comp.compare(t, result.get(loc)) > 0))
    			loc = loc + 1;
    		
		result.add(loc, t);
		return result;    		
    }
    
}
    