package sorting;
import java.util.*;

/**
 * One of the several sorting algorithms
 * to use for sorting detective work
 * @author Viera K. Proulx
 *
 * @param <T> the type of data to be sorted
 */
class SortingInsList<T> implements SortAlgorithm<T>{
	ILo<T> conslist;
	  
	  /**
	   * Initialize the list to be sorted from the given data.
	   * @param alist the <code>ArrayList</code> to sort
	   */
	  public void init(ArrayList<T> alistin){
	    this.conslist = new MtLo<T>();
	    for (T t: alistin){
	      this.conslist = new ConsLo<T>(t, this.conslist);
	    }
	  }
	
  /**
   * Sort the given <code>ArrayList</code> using the given 
   * <code>Comparator</code>. Return the sorted result.
   * For <em>in-place</em> algorithms <code>alist</code> should be
   * the result.
   * @param alist the <code>ArrayList</code> to sort
   * @param comp the <code>Comparator</code> that determines the ordering
   * @return the sorted <code>ArrayList</code>
   */
  public ArrayList<T> sort(Comparator<T> comp){
    ILo<T> result = this.conslist.insertionsort(comp);
    ArrayList<T> arlist = new ArrayList<T>();
    while(!(result instanceof MtLo)){
      arlist.add(((ConsLo<T>)result).first);
      result = ((ConsLo<T>)result).rest;
    }
    return arlist;
  }
  
}
    