package sorting;
import java.util.*;

/**
 * One of the several sorting algorithms
 * to use for sorting detective work
 * @author Viera K. Proulx
 *
 * @param <T> the type of data to be sorted
 */
class SortingSelection<T> implements SortAlgorithm<T>{

  /** The <code>ArrayList</code> to sort */
  public ArrayList<T> alist;
  
  /**
   * Initialize the list to be sorted from the given data.
   * @param alist the <code>ArrayList</code> to sort
   */
  public void init(ArrayList<T> alistin){
    this.alist = new ArrayList<T>();
    for (T t: alistin){
      this.alist.add(t);
    }
  }
  
  /**
   * Sort the given <code>ArrayList</code> using the given 
   * <code>Comparator</code>. Return the sorted result.
   * For <em>in-place</em> algorithms <code>alist</code> should be
   * the result.
   * @param alist the <code>ArrayList</code> to sort
   * @param comp the <code>Comparator</code> that determines the ordering
   * @return the sorted <code>ArrayList</code>
   */
  public ArrayList<T> sort(Comparator<T> comp){
    selectionsort(this.alist, comp);
    return this.alist;
  }
  
  /**
   * Sort the given list using selection sort
   * @param <T> The type of data in the <code>ArrayList</code>
   * @param alistthe <code>ArrayList</code> to sort
   * @param comp the <code>Comparator</code> that determines the ordering
   * @return
   */
  public void selectionsort(ArrayList<T> alist, Comparator<T> comp){    
    int i = 0;
    while (i < alist.size() - 1){
      swap(alist, i, findFirstLoc(alist, comp, i));
      i = i + 1;
    }       
  }
  
  /** 
   * swap the items at the given locations in the given array list
   * @param <T> The type of data in the <code>ArrayList</code>
   * @param a the <code>ArrayList</code> in which to swap
   * @param i one location to swap
   * @param jthe othrr location to swap
   */
  public void swap(ArrayList<T> a, int i, int j){
    T temp = a.get(i);
    temp = a.set(j, temp);
    a.set(i, temp);
  }
  
  /**
   * Find the location of the least element in the given <code>alist</code>
   * starting at the code>curr</code> location, using the given code>comp</code>
   * assume <code>curr < alist.size() - 2</code>
   * 
   * @param alist the list to sort
   * @param comp the <code>Comparator</code> that determines the ordering
   * @param curr the starting lowest location for unsorted part
   */
  public int findFirstLoc(ArrayList<T> alist, 
      Comparator<T> comp, int curr){
    int loc = curr;
    T minObj = alist.get(loc);
    int index = curr + 1;
    while (index < alist.size()){
      if (comp.compare(minObj, alist.get(index)) > 0){
        loc = index;
        minObj = alist.get(index);
      }
      index = index + 1;
    }
    return loc;
  }
}
    