package sorting;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;

import java.awt.*;

/**
 * A view for input of the desired timer test parameters.
 * 
 * @since 19 November 2009
 * @author Viera K. Proulx
 */
public class TimerInputView extends Display
implements JPTConstants{

  /** Display fields for selecting the algorithms. */
  protected BooleanView[] algoChoices;

  /** Display fields for selecting the data set sizes. */
  protected BooleanView[] setSizeChoices;

  /** Display fields for selecting the comparator to be used. */
  protected BooleanView[] comparatorChoices;
  
  protected boolean[][] timerInputs = new boolean[3][7];
  
  /** results of which algorithms are selected */
  boolean [] algos = new boolean[7];
  
  /** results of which sizes are selected */
  boolean [] sizes = new boolean[5];
  
  /** results of which comparators are selected */
  boolean [] comps = new boolean[3];

  /** The constructor */
  public TimerInputView() {
    super();

    // add the internal panel to the Display
    add(createDisplay());

    // set the title and annotation for the Display
    setTitleText("Timer tests");
  }

  //////////////////////////////////
  // Helpers for the Constructors //
  //////////////////////////////////

  /**
   * Construct a table panel display for algorithm choices,
   * sizes to test, and comparators to use
   */
  protected TablePanel createDisplay() {

    createViews();

    TablePanel timerDisplay = new TablePanel(new Object[]{
        new TablePanel(this.algoChoices, VERTICAL, 5, 5),
        new TablePanel(this.setSizeChoices, VERTICAL, 5, 5),
        new TablePanel(this.comparatorChoices, VERTICAL, 5, 5),},
        HORIZONTAL, 5, 5);

    return timerDisplay;        	
  }

  /**
   * Creates the panel for three input selectors: 
   * algorithm, data set size, and the comparator choices
   */
  protected void createViews() {

    // the collection of boolean views for selecting which algorithms to run
    this.algoChoices = new BooleanView[]{ 
    	new BooleanView("Sort 0"),
    	new BooleanView("Sort 1"),
    	new BooleanView("Sort 2"),
    	new BooleanView("Sort 3"),
    	new BooleanView("Sort 4"),
    	new BooleanView("Sort 5"),
      new BooleanView("Heapsort"),
    };

    // the collection of boolean views for selecting the data set sizes
    this.setSizeChoices = new BooleanView[]{ 
    	new BooleanView("2000"),
    	new BooleanView("4000"),
    	new BooleanView("8000"),
    	new BooleanView("16000"),
    	new BooleanView("29400")
    };
    
    // the collection of boolean views for selecting the comparators
    this.comparatorChoices = new BooleanView[]{ 
    	new BooleanView("By city name"),
    	new BooleanView("By longitude"),
    	new BooleanView("By zip code")
    };    
  }
  
  
  /**
   * Produce an array of algorithm selections
   */
  public boolean[] getAlgoChoices(){
	  for (int i = 0; i < 7; i++){
		  this.algos[i] = this.algoChoices[i].getBooleanValue();
	  }
	  return this.algos;
  }
  
  /**
   * Produce an array of size selections
   */
  public boolean[] getSizeChoices(){
	  for (int i = 0; i < 5; i++){
		  this.sizes[i] = this.setSizeChoices[i].getBooleanValue();
	  }
	  return this.sizes;
  }
  
  /**
   * Produce an array of algorithm selections
   */
  public boolean[] getCompChoices(){
	  for (int i = 0; i < 3; i++){
		  this.comps[i] = this.comparatorChoices[i].getBooleanValue();
	  }
	  return this.comps;
  }

  /**
   * Produce timer tests selections: algorithms to test,
   * data sizes to use, and the comparators to use.
   */
  public boolean[][] demandTimerData(){
    this.timerInputs[0] = this.getAlgoChoices();
    this.timerInputs[1] = this.getCompChoices();
    this.timerInputs[2] = this.getSizeChoices();
    return this.timerInputs;
  }

}