package sorting;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;

import tester.*;

import edu.neu.ccs.console.ConsoleAware;
import edu.neu.ccs.util.MathUtilities;

import student.*;

/**
 * 
 * @author Viera K. Proulx
 */
class TimerTests implements ConsoleAware{

  Algorithms base = new Algorithms();

  // an arraylist of sorting algorithms to test	
  SortAlgorithm<City> bst   = new SortingBST<City>();
  SortAlgorithm<City> insertlist  = new SortingInsList<City>();
  SortAlgorithm<City> selectarray = new SortingSelection<City>();
  SortAlgorithm<City> quicklist   = new SortingQuicksortList<City>();
  SortAlgorithm<City> insertarray = new SortingIns<City>();
  SortAlgorithm<City> quickarray  = new SortingQuicksort<City>();
  SortAlgorithm<City> heapsort  = new student.Heapsort<City>();

  ArrayList<SortAlgorithm<City>> sortAlgosAll =
    new ArrayList<SortAlgorithm<City>>();

  /** 
   * initialize the <code>ArrayList</code> 
   * of <code>SortAlgorithm</code>s to use
   */
  void initAlgos(){
    this.sortAlgosAll.add(this.bst);
    this.sortAlgosAll.add(this.insertlist);
    this.sortAlgosAll.add(this.selectarray);
    this.sortAlgosAll.add(this.quicklist);
    this.sortAlgosAll.add(this.insertarray);
    this.sortAlgosAll.add(this.quickarray);
    this.sortAlgosAll.add(this.heapsort);
  }

  ArrayList<SortAlgorithm<City>> sortAlgos = 
    new ArrayList<SortAlgorithm<City>>();
  ArrayList<Integer> sortAlgosIndex = 
    new ArrayList<Integer>();

  Comparator<City> byName      = new ByName();
  Comparator<City> byLongitude = new ByLongitude();
  Comparator<City> byZip       = new ByZipCode();

  ArrayList<Comparator<City>> comparatorsAll = 
    new ArrayList<Comparator<City>>();
  
  /** 
   * initialize the <code>ArrayList</code> 
   * of <code>Comparator</code>s to use 
   */
  void initComparators(){
    this.comparatorsAll.add(this.byName);
    this.comparatorsAll.add(this.byLongitude);
    this.comparatorsAll.add(this.byZip);
  }

  // an arraylist of comparators to use
  ArrayList<Comparator<City>> comparators = new ArrayList<Comparator<City>>();
  ArrayList<Integer> comparatorsIndex = new ArrayList<Integer>();

  ArrayList<Integer> sizesAll = 
    new ArrayList<Integer>();

  /** 
   * initialize the <code>ArrayList</code> 
   * of data sizes to use
   */
  void initSizes(){
    this.sizesAll.add(2000);
    this.sizesAll.add(4000);
    this.sizesAll.add(8000);
    this.sizesAll.add(16000);
    this.sizesAll.add(29000);
  }

  ArrayList<Integer> dataSizes = new ArrayList<Integer>();
  ArrayList<Integer> datasizesIndex = new ArrayList<Integer>();

  // original data set of all cities available
  ArrayList<City> alldata;
  ArrayList<City> randomdata;
  ILo<City> alldataList;

  // records of the start and end time
  long startTime;			// time of the start of the test
  long endTime;			// time of the end of the test

  /**
   * Constructor that initializes all algorithms, 
   * reads the data for all cities.
   */
  TimerTests(){
    this.base = new Algorithms();
    this.base.initCities(30000);
    this.alldata = this.base.cities;
    this.randomdata = this.base.randomData();
    this.alldataList = this.base.citylist;
  }

  /**
   * set up a database of all algorithms, comparators, 
   * and possible sizes, 
   * then select the ones to be used in current tests
   */
  public void initAll(){
    // set up the permanent data
    initAlgos();
    initComparators();
    initSizes();

    GUITimerInput choices = new GUITimerInput();
    boolean[][] timerParams = choices.demandTimerInputs();

    // make the current selections
    initAlgos(timerParams[0]);
    initComparators(timerParams[1]);
    initSizes(timerParams[2]);
  }

  /**
   * Select the given algorithms for tests
   * @param picks a boolean array of algorithms to choose
   */
  public void initAlgos(boolean[] picks){
    this.sortAlgos.clear();
    this.sortAlgosIndex.clear();
    for (int i = 0; i < 7; i++){
      if (picks[i]){
        this.sortAlgos.add(this.sortAlgosAll.get(i));
        this.sortAlgosIndex.add(i);
      }
    }
  }

  /**
   * Select the given comparators for tests
   * @param picks a boolean array of comparators to choose
   */
  public void initComparators(boolean[] picks){
    this.comparators.clear();
    this.comparatorsIndex.clear();
    for (int i = 0; i < 3; i++){
      if (picks[i]){
        this.comparators.add(this.comparatorsAll.get(i));
        this.comparatorsIndex.add(i);
      }
    }
  }

  /**
   * Select the given data sizes for tests
   * @param picks a boolean array of data sizes to choose
   */
  public void initSizes(boolean[] picks){
    this.dataSizes.clear();
    this.datasizesIndex.clear();
    for (int i = 0; i < 5; i++){
      if (picks[i]){
        this.dataSizes.add(this.sizesAll.get(i));
        this.datasizesIndex.add(i);
      }
    }
  }

  /**
   * The actual runner for all tests
   */
  public void runAllTests(){
    for (Integer i: this.sortAlgosIndex){
      System.out.println("Algorithm " + i);
    }

    // print the header
    console.out.println("\nAlgorithm, \tNo., \tComparator, \tSize, \tRunning time");

    long runtime;

    for (int i = 0; i < this.sortAlgos.size(); i++)
      for (int j = 0; j < this.comparators.size(); j++)
        for (int k = 0; k < this.dataSizes.size(); k++){
          runtime = this.runOneTest(this.sortAlgos.get(i),
              this.comparators.get(j),
              this.dataSizes.get(k));

          console.out.println("Algorithm, \t" + 
              this.sortAlgosIndex.get(i) + ", \t" + 
              this.comparators.get(j).getClass().getName().substring(8) + ", \t" +
              this.dataSizes.get(k) + ", \t" + runtime);
        }

    runtime = this.runOneTest(
        new SortingBST<City>(),
        this.byName,
        10000);

  }

  /**
   * run one test of a given sorting algorithm
   * with the given comparator and the desired size of data
   * @param algo the given sorting algorithm
   * @param comp the given comparator
   * @param size the desired size of data
   * @return
   */
  public long runOneTest(SortAlgorithm<City> algo, 
      Comparator<City> comp, 
      int size) {

    // construct an iterator to produce the desired data set
    ArrayList<City> source = new ArrayList<City>();
    ArrayList<City> result;

    for (int i = 0; i < size; i++){
      source.add(this.randomdata.get(i));
    }
    algo.init(source);
    this.startTime = (new Date()).getTime();
    result = algo.sort(comp);
    this.endTime = (new Date()).getTime();
    if (result.size() == source.size())
      return this.endTime - this.startTime;
    else
      return -1; // not all data has been sorted
  }

  /**
   * A sample run of the timing tests
   */
  public void sampleTests(){
    long runtime;

    runtime = this.runOneTest(
        new SortingBST<City>(),
        this.byName,
        10000);

    console.out.println("Algorithm 0 time: " + runtime + "\n");

    runtime = this.runOneTest(
        new SortingInsList<City>(),
        this.byName, 
        10000);

    console.out.println("Algorithm 1 time: " + runtime + "\n");


    runtime = this.runOneTest(
        new SortingSelection<City>(),
        this.byName, 
        1000);

    console.out.println("Algorithm 2 time: " + runtime + "\n");


    runtime = this.runOneTest(
        new SortingQuicksort<City>(),
        this.byName, 
        10000);

    console.out.println("Algorithm 3 time: " + runtime + "\n");


    runtime = this.runOneTest(
        new SortingIns<City>(),
        this.byName, 
        10000);

    console.out.println("Algorithm 4 time: " + runtime + "\n");



    runtime = this.runOneTest(
        new SortingQuicksortList<City>(),
        new Comparator<City>(){
          public int compare(City city1, City city2){
            return city1.name.compareTo(city2.name);
          }			
        }, 
        10000);

    console.out.println("Algorithm 5 time: " + runtime + "\n");
    

    runtime = this.runOneTest(
        new Heapsort<City>(),
        new Comparator<City>(){
          public int compare(City city1, City city2){
            return city1.name.compareTo(city2.name);
          }     
        }, 
        10000);

    console.out.println("Algorithm 6 time: " + runtime + "\n");

  }

  public static void main(String[] argv){

    TimerTests tt = new TimerTests();
    tt.sampleTests();
  }
}