package student;

import java.util.*;
import tester.*;

public class Heapsort<T> extends SortingHeapSort<T>{
  
  public ArrayList<T> heapsort(ArrayList<T> alist, Comparator<T> comp) {
    heapify(alist, comp);
    destroyheap(alist, comp);
    return alist;
  }
  
  public ArrayList<T> destroyheap(ArrayList<T> alist, Comparator<T> comp) {
    int lastnode = alist.size() - 1;
    while(lastnode > 1){
      //swap(alist.get(1), alist.get(lastnode));
      alist.set(1, alist.set(lastnode, alist.get(1)));
      lastnode = lastnode - 1;
      downheapTo(alist, comp, 1, lastnode + 1);
    }
    return alist;
  }
  
  // convert the given arraylist into a heap
  public ArrayList<T> heapify(ArrayList<T> alist, Comparator<T> comp) {
   int lastnode = alist.size()/2;
   for (int k = lastnode; k > 0; k--){
     downheapTo(alist, comp, k, alist.size());
   }
   return alist;
  }
 
  // downheap from the given node - adjusting the heap below
  public ArrayList<T> downheapTo(ArrayList<T> alist, 
      Comparator<T> comp, int k, int heapsize) {
  
    if (2 * k + 1 > heapsize)
      return alist;
    else {
      int ci = biggerchild(alist, comp, k, heapsize);
      if (comp.compare(alist.get(ci), alist.get(k)) > 0){
        alist.set(k, alist.set(ci, alist.get(k)));
        downheapTo(alist, comp, ci, heapsize);
      } 
    }
    return alist;
  }

  // find the bigger child of the given node
  public int biggerchild(ArrayList<T> alist, 
      Comparator<T> comp, int k, int heapsize) {
    if (2*k + 1 == heapsize)
      return 2*k;
    else{
      if (comp.compare(alist.get(2*k), alist.get(2*k+1)) > 0)
        return 2*k;
      else
        return 2*k+1;
    }  
  }
}

class ExamplesHeapsort{
  ExamplesHeapsort(){}
  
  ArrayList<Integer> heap1 = new ArrayList<Integer>();
  ArrayList<Integer> heap2 = new ArrayList<Integer>();
  ArrayList<Integer> heap3 = new ArrayList<Integer>();
  
  ArrayList<Integer> noheap = new ArrayList<Integer>();
  ArrayList<Integer> makeheap = new ArrayList<Integer>();
  
  ArrayList<Integer> starter = new ArrayList<Integer>();
  ArrayList<Integer> ender = new ArrayList<Integer>();
  ArrayList<Integer> sorted = new ArrayList<Integer>();
  ArrayList<Integer> unsorted = new ArrayList<Integer>();
  
  void initHeap(){
    this.heap1.add(null);
    this.heap1.add(80);
    this.heap1.add(50);
    this.heap1.add(40);
    this.heap1.add(45);
    this.heap1.add(20);
    this.heap1.add(30);

    this.heap2.add(null);
    this.heap2.add(50);
    this.heap2.add(45);
    this.heap2.add(40);
    this.heap2.add(30);
    this.heap2.add(20);

    this.heap3.add(null);
    this.heap3.add(70);
    this.heap3.add(45);
    this.heap3.add(50);
    this.heap3.add(30);
    this.heap3.add(20);
    this.heap3.add(40);
    

    this.noheap.add(null);
    this.noheap.add(40);
    this.noheap.add(60);
    this.noheap.add(20);
    this.noheap.add(50);

    this.makeheap.add(null);
    this.makeheap.add(60);
    this.makeheap.add(50);
    this.makeheap.add(20);
    this.makeheap.add(40);
    
    this.starter.add(null);
    this.starter.add(20);
    this.starter.add(50);
    this.starter.add(40);
    this.starter.add(30);
    this.starter.add(60);
    this.starter.add(70);
    
    this.ender.add(null);
    this.ender.add(70);
    this.ender.add(60);
    this.ender.add(40);
    this.ender.add(30);
    this.ender.add(50);
    this.ender.add(20);
    
    this.sorted.add(null);
    this.sorted.add(20);
    this.sorted.add(30);
    this.sorted.add(40);
    this.sorted.add(50);
    this.sorted.add(60);
    this.sorted.add(70);
    
    this.unsorted.add(null);
    this.unsorted.add(70);
    this.unsorted.add(30);
    this.unsorted.add(60);
    this.unsorted.add(20);
    this.unsorted.add(50);
    this.unsorted.add(40);
  }
  

  void clearHeap(){
    this.heap1.clear();
    this.heap2.clear();
    this.heap3.clear();
    this.noheap.clear();
    this.makeheap.clear();
    this.starter.clear();
    this.ender.clear();
    this.sorted.clear();
    this.unsorted.clear();
  }
  
  Comparator<Integer> intcomp = new Comparator<Integer>(){
    public int compare(Integer i1, Integer i2){
      return i1 - i2;
    }
  };
  
  Heapsort<Integer> hs = new Heapsort<Integer>();
  
  void printlist(ArrayList<Integer> inlist){
    System.out.print("list: ");
    for (Integer i: inlist)
      System.out.print(i + " ");
    System.out.println();
  }
  
  void testBiggerChild(Tester t){
    initHeap();
    t.checkExpect(this.hs.biggerchild(this.heap3, this.intcomp, 1, this.heap3.size()), 3);
    t.checkExpect(this.hs.biggerchild(this.heap3, this.intcomp, 2, this.heap3.size()), 4);
    t.checkExpect(this.hs.biggerchild(this.heap3, this.intcomp, 3, this.heap3.size()), 6);
    t.checkExpect(this.hs.biggerchild(this.heap2, this.intcomp, 1, this.heap3.size()), 2);
    clearHeap(); 
  }  
 
  void testDownheap(Tester t){
    initHeap();
    this.hs.downheapTo(this.noheap, this.intcomp, 1, this.noheap.size());
    t.checkExpect(this.noheap, this.makeheap);
    clearHeap(); 
  } 
  
  void testHeapify(Tester t){
    initHeap();
    this.hs.heapify(this.starter, this.intcomp);
    t.checkExpect(this.starter, this.ender);
    this.hs.destroyheap(this.ender, this.intcomp);
    t.checkExpect(this.ender, this.sorted);
    clearHeap(); 
  }
 /* 
  void testDestroyheap(Tester t){
    initHeap();
    this.hs.destroyheap(this.ender, this.intcomp);
    t.checkExpect(this.ender, this.sorted);
    clearHeap(); 
  }
  
  void testHeapsort(Tester t){
    initHeap();
    printlist(this.unsorted);
    this.hs.heapify(this.unsorted, this.intcomp);
    printlist(this.unsorted);
    this.hs.destroyheap(this.unsorted, this.intcomp);
    printlist(this.unsorted);
    
    clearHeap();
    initHeap();
    this.hs.heapsort(this.unsorted, this.intcomp);
    t.checkExpect(this.unsorted, this.sorted);
    clearHeap(); 
  }
  */
}